==> python_pptx_interface-0.0.1-py3-none-any.whl: pptx_tools/templates.py <==
"""
This file contains variables with names of important pptx template master_slide shapes
"""
from datetime import datetime

# from pptx.enum.text import MSO_AUTO_SIZE
from pptx import Presentation

from pptx_tools.better_abc import ABCMeta, abstract_attribute


class AbstractTemplate(metaclass=ABCMeta):
    """
    Templates should subclass this abstract class, to make sure,
    that all important attributes/methodes are defined.
    """
    @abstract_attribute
    def TEMPLATE_FILE(cls):
        pass

    @abstract_attribute
    def prs(self):
        pass

    @abstract_attribute
    def title_layout(cls):
        pass

    @abstract_attribute
    def default_layout(cls):
        pass


# ----------------------------------------------------------------------------
# --------- Customized template classes are needed for each template ---------
# ----------------------------------------------------------------------------
class TemplateExample(AbstractTemplate):
    """
    Class handling example-template.pptx).
    """
    TEMPLATE_FILE = '..\\resources\pptx_template\\example-template.pptx'

    def __init__(self):
        self.prs = Presentation(self.TEMPLATE_FILE)

        self.title_layout = self.prs.slide_masters[0].slide_layouts[0]
        self.default_layout = self.prs.slide_masters[1].slide_layouts[0]

        # following names are the same for both slide masters
        self.author_shape_name = "Rectangle 4"
        self.website_shape_name = "Rectangle 5"

        date_time = datetime.now().strftime("%d %B, %Y")
        self.set_author("Nathanael Jöhrmann", city="Chemnitz", date=date_time)
        self.set_website("https://github.com/natter1/pi88reader")

    def set_author(self, name, city=None, date=None):
        text = ""
        spacer = " ∙ "
        if city:
            text += city + spacer
        if date:
            text += date + spacer
        text += name
        self.write_text_to_master_shape(text=text, shape_name=self.author_shape_name)

    def set_website(self, text):
        self.write_text_to_master_shape(text = text, shape_name=self.website_shape_name)

    def write_text_to_master_shape(self, text, shape_name):
        for shape in self.master_shapes:
            if not shape.has_text_frame:
                continue
            if shape.name == shape_name:
                change_paragraph_text_to(shape.text_frame.paragraphs[0], text)
    @property
    def master_shapes(self):
        result = []
        for slide_master in self.prs.slide_masters:
            result.extend(slide_master.shapes)
        return result


# ETIT_16-9.pptx is not part of the repository due to legal restrictions!
class TemplateETIT169(TemplateExample):
    """
    Class handling ETIT 16:9 template.
    """
    TEMPLATE_FILE = '..\\resources\pptx_template\\ETIT_16-9.pptx'

    def __init__(self):
        super().__init__()
        date_time = datetime.now().strftime("%d %B, %Y")
        self.set_author("Nathanael Jöhrmann", city="Chemnitz", date=date_time)
        self.set_website("https://www.tu-chemnitz.de/etit/wetel/")


def analyze_pptx(template_file):
    """ Take the given file and analyze the structure of master slides.
    Prints shape names/ids and texts for SlideMaster-shapes
    To get an output file contains marked up information
    remove comment on last two lines of function.
    This is helpful when manipulating template-files.
    """
    prs = Presentation(template_file)
    # Each powerpoint file has multiple layouts
    # Loop through them all and  see where the various elements are
    slide_masters = prs.slide_masters
    for index, slide_master in enumerate(prs.slide_masters):
        print('------------------------------------')
        print('------------------------------------')
        print(f"slide master indexed: {index}")
        print(slide_master)
        print("text boxes:")
        for shape in slide_master.shapes:
            try:
                dummystring = f"shape name: {shape.name} - shape text: {shape.text}"
                shape.text = shape.name
                print(dummystring)
            except:
                pass
            #shape.text = 'hahahaha'
        # for shape in slide_master.slideshapes:
        #     print(shape)
        print('------------------------------------')
        for index, slide_layout in enumerate(slide_master.slide_layouts):
            print(f"\tslide layout: {slide_layout.name}")
            slide = prs.slides.add_slide(slide_master.slide_layouts[index])
            # Not every slide has to have a title
            try:
                title = slide.shapes.title
                title.text = 'Title for Layout {}'.format(index)
            except AttributeError:
                print("No Title for Layout {}".format(index))
            # Go through all the placeholders and identify them by index and type
            for shape in slide.placeholders:
                if shape.is_placeholder:
                    phf = shape.placeholder_format
                    # Do not overwrite the title which is just a special placeholder
                    try:
                        if 'Title' not in shape.text:
                            shape.text = 'Placeholder index:{} type:{}'.format(phf.idx, shape.name)
                    except AttributeError:
                        print("{} has no text attribute".format(phf.type))
                    print(f"\t\tid: {phf.idx} - name: {shape.name}")
    # output_file = '..\\resources\pptx\\template_names.pptx'
    # prs.save(output_file)


# todo: add something like this directly to python-pptx?
def copy_font(_from, _to):
    _to.bold = _from.bold
    # todo: color is ColorFormat object
    # _to.set_color = _from.color
    # todo: fill is FillFormat object
    # _to.fill = _from.fill
    _to.italic = _from.italic
    _to.language_id = _from.language_id
    _to.name = _from.name
    _to.size = _from.size
    _to.underline = _from.underline


def change_paragraph_text_to(paragraph, text):
    """
    Change text of paragraph to text, but keep format.
    :param paragraph:
    :param text:
    :return:
    """
    font = paragraph.runs[0].font
    paragraph.text = text
    copy_font(_from=font, to=paragraph.runs[0].font)


def analyze_paragraphs(paragraphs):
    for index, para in enumerate(paragraphs):
        print(f"index: {index} - text: {para.text}")
        for run_index, run in enumerate(para.runs):
            print(f"\trun: {run.text}")

==> python_pptx_interface-0.0.2-py3-none-any.whl: pptx_tools/templates.py <==
"""
This file contains variables with names of important pptx template master_slide shapes
"""
from datetime import datetime
import pkg_resources

# from pptx.enum.text import MSO_AUTO_SIZE
from pptx import Presentation

from pptx_tools.better_abc import ABCMeta, abstract_attribute


class AbstractTemplate(metaclass=ABCMeta):
    """
    Templates should subclass this abstract class, to make sure,
    that all important attributes/methodes are defined.
    """
    @abstract_attribute
    def TEMPLATE_FILE(cls):
        pass

    @abstract_attribute
    def prs(self):
        pass

    @abstract_attribute
    def title_layout(cls):
        pass

    @abstract_attribute
    def default_layout(cls):
        pass


# ----------------------------------------------------------------------------
# --------- Customized template classes are needed for each template ---------
# ----------------------------------------------------------------------------
class TemplateExample(AbstractTemplate):
    """
    Class handling example-template.pptx).
    """
    TEMPLATE_FILE = pkg_resources.resource_filename('pptx_tools', 'resources/example-template.pptx')

    def __init__(self):
        self.prs = Presentation(self.TEMPLATE_FILE)

        self.title_layout = self.prs.slide_masters[0].slide_layouts[0]
        self.default_layout = self.prs.slide_masters[1].slide_layouts[0]

        # following names are the same for both slide masters
        self.author_shape_name = "Rectangle 4"
        self.website_shape_name = "Rectangle 5"

        date_time = datetime.now().strftime("%d %B, %Y")
        self.set_author("Nathanael Jöhrmann", city="Chemnitz", date=date_time)
        self.set_website("https://github.com/natter1/pi88reader")

    def set_author(self, name, city=None, date=None):
        text = ""
        spacer = " ∙ "
        if city:
            text += city + spacer
        if date:
            text += date + spacer
        text += name
        self.write_text_to_master_shape(text=text, shape_name=self.author_shape_name)

    def set_website(self, text):
        self.write_text_to_master_shape(text = text, shape_name=self.website_shape_name)

    def write_text_to_master_shape(self, text, shape_name):
        for shape in self.master_shapes:
            if not shape.has_text_frame:
                continue
            if shape.name == shape_name:
                change_paragraph_text_to(shape.text_frame.paragraphs[0], text)
    @property
    def master_shapes(self):
        result = []
        for slide_master in self.prs.slide_masters:
            result.extend(slide_master.shapes)
        return result


def analyze_pptx(template_file):
    """ Take the given file and analyze the structure of master slides.
    Prints shape names/ids and texts for SlideMaster-shapes
    To get an output file contains marked up information
    remove comment on last two lines of function.
    This is helpful when manipulating template-files.
    """
    prs = Presentation(template_file)
    # Each powerpoint file has multiple layouts
    # Loop through them all and  see where the various elements are
    slide_masters = prs.slide_masters
    for index, slide_master in enumerate(prs.slide_masters):
        print('------------------------------------')
        print('------------------------------------')
        print(f"slide master indexed: {index}")
        print(slide_master)
        print("text boxes:")
        for shape in slide_master.shapes:
            try:
                dummystring = f"shape name: {shape.name} - shape text: {shape.text}"
                shape.text = shape.name
                print(dummystring)
            except:
                pass
            #shape.text = 'hahahaha'
        # for shape in slide_master.slideshapes:
        #     print(shape)
        print('------------------------------------')
        for index, slide_layout in enumerate(slide_master.slide_layouts):
            print(f"\tslide layout: {slide_layout.name}")
            slide = prs.slides.add_slide(slide_master.slide_layouts[index])
            # Not every slide has to have a title
            try:
                title = slide.shapes.title
                title.text = 'Title for Layout {}'.format(index)
            except AttributeError:
                print("No Title for Layout {}".format(index))
            # Go through all the placeholders and identify them by index and type
            for shape in slide.placeholders:
                if shape.is_placeholder:
                    phf = shape.placeholder_format
                    # Do not overwrite the title which is just a special placeholder
                    try:
                        if 'Title' not in shape.text:
                            shape.text = 'Placeholder index:{} type:{}'.format(phf.idx, shape.name)
                    except AttributeError:
                        print("{} has no text attribute".format(phf.type))
                    print(f"\t\tid: {phf.idx} - name: {shape.name}")
    # output_file = '..\\resources\pptx\\template_names.pptx'
    # prs.save(output_file)


# todo: add something like this directly to python-pptx?
def copy_font(_from, _to):
    _to.bold = _from.bold
    # todo: color is ColorFormat object
    # _to.set_color = _from.color
    # todo: fill is FillFormat object
    # _to.fill = _from.fill
    _to.italic = _from.italic
    _to.language_id = _from.language_id
    _to.name = _from.name
    _to.size = _from.size
    _to.underline = _from.underline


def change_paragraph_text_to(paragraph, text):
    """
    Change text of paragraph to text, but keep format.
    :param paragraph:
    :param text:
    :return:
    """
    font = paragraph.runs[0].font
    paragraph.text = text
    copy_font(_from=font, _to=paragraph.runs[0].font)


def analyze_paragraphs(paragraphs):
    for index, para in enumerate(paragraphs):
        print(f"index: {index} - text: {para.text}")
        for run_index, run in enumerate(para.runs):
            print(f"\trun: {run.text}")
